#version 400

in vec3 inputPosition;
in vec2 inputTexCoord;
in vec3 inputNormal;
in vec4 skinWeights;
in vec4 skinIndices;

out vec2 texCoord;
out vec3 normal;

uniform mat4 worldMatrix;
uniform mat4 viewMatrix;
uniform mat4 projectionMatrix;
uniform mat4 bones[61];

void main(void)
{
	vec4 newVertex = vec4(inputPosition, 1.0f);
	vec3 newNormal = inputNormal;

	if(skinWeights != vec4(0.0f,0.0f,0.0f,0.0f))
	{
		vec4 tempNormal = vec4(newNormal, 0.0f);

		int index1 = int(skinIndices.x);
		int index2 = int(skinIndices.y);
		int index3 = int(skinIndices.z);
		int index4 = int(skinIndices.w);

		mat4 matrix1 = bones[index1] * skinWeights.x;
		mat4 matrix2 = bones[index2] * skinWeights.y;
		mat4 matrix3 = bones[index3] * skinWeights.z;
		mat4 matrix4 = bones[index4] * skinWeights.w;

		mat4 sum_matrix = matrix1 + matrix2 + matrix3 + matrix4;
		float sum_weight = skinWeights.x + skinWeights.y + skinWeights.z + skinWeights.w;

		newVertex = sum_matrix * newVertex;
		newVertex = newVertex / sum_weight;

		tempNormal = sum_matrix * tempNormal;
		tempNormal = tempNormal / sum_weight;
	
		newNormal.x = tempNormal.x ;
		newNormal.y = tempNormal.y;
		newNormal.z = tempNormal.z;
	}

	gl_Position = worldMatrix * newVertex;
	gl_Position = viewMatrix * gl_Position;
	gl_Position = projectionMatrix * gl_Position;			
	
	texCoord = inputTexCoord;
							
	normal = mat3(worldMatrix) * newNormal;
	normal = normalize(normal);
}